@@ -393,6 +393,32 @@ Otherwise, it saves all modified buffers without asking."
   :type 'string
   :group 'compilation)
 
+;; The system null device. (Should reference NULL_DEVICE from C.)
+(defvar grep-null-device "/dev/null" "The system null device.")
+
+(defvar grep-find-use-xargs
+  (if (equal (call-process "find" nil nil nil
+			   grep-null-device "-print0")
+	     0)
+      'gnu)
+  "Whether \\[grep-find] uses the `xargs' utility by default.
+
+If nil, it uses `grep -exec'; if `gnu', it uses `find -print0' and `xargs -0';
+if not nil and not `gnu', it uses `find -print' and `xargs'.
+
+This variable's value takes effect when `compile.el' is loaded
+by influencing the default value for the variable `grep-find-command'.")
+
+(defvar grep-find-command
+  (cond ((eq grep-find-use-xargs 'gnu)
+	 (format "find . -type f -print0 | xargs -0 -e %s" grep-command))
+	(grep-find-use-xargs
+	 (format "find . -type f -print | xargs %s" grep-command))
+	(t (cons (format "find . -type f -exec %s {} /dev/null \\;"
+			 grep-command)
+		 (+ 22 (length grep-command)))))
+  "The default find command for \\[grep-find].")
+
 ;;;###autoload
 (defcustom compilation-search-path '(nil)
   "*List of directories to search for source files named in error messages.
@@ -446,6 +472,7 @@ write into the compilation buffer, and to put in its mode line.")
 (defvar compile-history nil)
 ;; History of grep commands.
 (defvar grep-history nil)
+(defvar grep-find-history nil)
 
 ;; XEmacs
 (defvar compilation-font-lock-keywords (purecopy
@@ -522,9 +549,6 @@ to a function that generates a unique name."
   (save-some-buffers (not compilation-ask-about-save) nil)
   (compile-internal compile-command "No more errors"))
 
-;; The system null device. (Should reference NULL_DEVICE from C.)
-(defvar grep-null-device "/dev/null" "The system null device.")
-
 ;;;###autoload
 (defun grep (command-args)
   "Run grep, with user-specified args, and collect output in a buffer.
@@ -562,6 +586,20 @@ easily repeat a grep command."
 			  (cons msg code)))
 		 (cons msg code))))))))
 
+(defun grep-find (command-args)
+  "Run grep via find, with user-specified args, and collect output in a buffer.
+While find runs asynchronously, you can use the \\[next-error] command
+to find the text that grep hits refer to.
+
+This command uses a special history list for its arguments, so you can
+easily repeat a find command."
+  (interactive
+   ;; XEmacs change..
+   (list (read-shell-command "Run find (like this): "
+			       grep-find-command 'grep-find-history)))
+  (let ((grep-null-device nil))		; see grep
+    (grep command-args)))
+
 (defun compile-internal (command error-message
 				 &optional name-of-mode parser regexp-alist
 				 name-function)
@@ -689,7 +727,13 @@ Returns the compilation buffer created."
 ;; Set the height of WINDOW according to compilation-window-height.
 (defun compilation-set-window-height (window)
   (and compilation-window-height
-       (= (window-width window) (frame-width (window-frame window)))
+       ;; Check if the window is split horizontally.
+       ;; Emacs checks window width versus frame-width:
+       ;;   (= (window-width window) (frame-width (window-frame window)))
+       ;; But XEmacs must take into account a possible left or right
+       ;; toolbar:
+       (and (window-leftmost-p (selected-window))
+	    (window-rightmost-p (selected-window)))
        ;; If window is alone in its frame, aside from a minibuffer,
        ;; don't change its height.
        (not (eq window (frame-root-window (window-frame window))))
@@ -1574,6 +1618,23 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 	  (goto-char (cdr next-error))))))
     
 
+;; helper routine to compilation-find-file
+(defun compare-file-to-buffer (filename buffer)
+  "compare BUFFER to FILENAME and see if they are really the same. Returns nil
+if not, BUFFER if they are"
+  (interactive "FCompare FILE : 
+bTo BUFFER : ")
+  ;; (print (format "%s %s " buffer filename))
+  (if (and (stringp filename) (bufferp buffer))
+      (save-excursion
+	(set-buffer buffer)
+	(if (equal (file-truename (buffer-file-name))
+		   (file-truename filename))
+	    buffer
+	  nil))
+    nil)
+)
+
 ;; Find a buffer for file FILENAME.
 ;; Search the directories in compilation-search-path.
 ;; A nil in compilation-search-path means to try the
@@ -1581,44 +1642,50 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 ;; If FILENAME is not found at all, ask the user where to find it.
 ;; Pop up the buffer containing MARKER and scroll to MARKER if we ask the user.
 (defun compilation-find-file (marker filename dir &rest formats)
-  (or formats (setq formats '("%s")))
-  (let ((dirs compilation-search-path)
-	buffer thisdir fmts name)
-    (if (file-name-absolute-p filename)
-	;; The file name is absolute.  Use its explicit directory as
-	;; the first in the search path, and strip it from FILENAME.
-	(setq filename (abbreviate-file-name (expand-file-name filename))
-	      dirs (cons (file-name-directory filename) dirs)
-	      filename (file-name-nondirectory filename)))
-    ;; Now search the path.
-    (while (and dirs (null buffer))
-      (setq thisdir (or (car dirs) dir)
-	    fmts formats)
-      ;; For each directory, try each format string.
-      (while (and fmts (null buffer))
-	(setq name (expand-file-name (format (car fmts) filename) thisdir)
-	      buffer (and (file-exists-p name)
-			  (find-file-noselect name))
-	      fmts (cdr fmts)))
-      (setq dirs (cdr dirs)))
-    (or buffer
-	;; The file doesn't exist.
-	;; Ask the user where to find it.
-	;; If he hits C-g, then the next time he does
-	;; next-error, he'll skip past it.
-	(let* ((pop-up-windows t)
-	       (w (display-buffer (marker-buffer marker))))
-	  (set-window-point w marker)
-	  (set-window-start w marker)
-	  (let ((name (expand-file-name
-		       (read-file-name
-			(format "Find this error in: (default %s) "
-				filename)
-			dir filename t))))
-	    (if (file-directory-p name)
-		(setq name (expand-file-name filename name)))
-	    (and (file-exists-p name)
-		 (find-file-noselect name)))))))
+    (or formats (setq formats '("%s")))
+    (let* ((dirs compilation-search-path)
+	   (true-file-name (file-name-nondirectory filename))
+	   (buffer (get-buffer true-file-name)) 
+	   fi thisdir fmts name)
+      ;; first see if buffer is there already
+      (if (compare-file-to-buffer buffer filename)
+	  (switch-to-buffer buffer)
+	(progn
+	  (if (file-name-absolute-p filename)
+	      ;; The file name is absolute.  Use its explicit directory as
+	      ;; the first in the search path, and strip it from FILENAME.
+	      (setq filename (abbreviate-file-name (expand-file-name filename))
+		    dirs (cons (file-name-directory filename) dirs)
+		    filename (file-name-nondirectory filename)))
+	  ;; Now search the path.
+	  (while (and dirs (null buffer))
+	    (setq thisdir (or (car dirs) dir)
+		  fmts formats)
+	    ;; For each directory, try each format string.
+	    (while (and fmts (null buffer))
+	      (setq name (expand-file-name (format (car fmts) filename) thisdir)
+		    buffer (and (file-exists-p name)
+				(find-file-noselect name))
+		    fmts (cdr fmts)))
+	    (setq dirs (cdr dirs)))
+	  (or buffer
+	      ;; The file doesn't exist.
+	      ;; Ask the user where to find it.
+	      ;; If he hits C-g, then the next time he does
+	      ;; next-error, he'll skip past it.
+	      (let* ((pop-up-windows t)
+		     (w (display-buffer (marker-buffer marker))))
+		(set-window-point w marker)
+		(set-window-start w marker)
+		(let ((name (expand-file-name
+			     (read-file-name
+			      (format "Find this error in: (default %s) "
+				      filename)
+			      dir filename t))))
+		  (if (file-directory-p name)
+		    (setq name (expand-file-name filename name)))
+		  (and (file-exists-p name)
+		       (find-file-noselect name)))))))))
 
 ;; Set compilation-error-list to nil, and unchain the markers that point to the
 ;; error messages and their text, so that they no longer slow down gap motion.
